@@ -5,7 +5,8 @@ namespace PlasmaScripting {
     using Qyoto;
     using Plasma;
 
-    public class QGraphicsWidget : QObject, IQGraphicsItem {
+    public class QGraphicsWidget : QObject{
+//    public class QGraphicsWidget : QObject, IQGraphicsItem {
         protected AppletScript appletScript;
         protected Plasma.Applet applet;
         protected Type appletType;
@@ -15,31 +16,31 @@ namespace PlasmaScripting {
         public Plasma.Applet PlasmaApplet {
             get { return applet; }
         }
-        [Q_PROPERTY("QPalette", "palette")]
-        public QPalette Palette {
-            get { return applet.Palette; }
-            set { applet.Palette = value; }
-        }
+//        [Q_PROPERTY("QPalette", "palette")]
+//        public QPalette Palette {
+//            get { return applet.Palette; }
+//            set { applet.Palette = value; }
+//        }
 //        [Q_PROPERTY("QFont", "font")]
 //        public QFont Font {
-//            get { return applet.Font; }
-//            set { applet.Font = value; }
+//            get { return ((QyotoQGraphicsWidget) applet).Font; }
+//            set { ((QyotoQGraphicsWidget) applet).Font = value; }
+//        }
+//        [Q_PROPERTY("Qt::LayoutDirection", "layoutDirection")]
+//        public new Qt.LayoutDirection LayoutDirection {
+//            get { return applet.LayoutDirection; }
+//            set { applet.LayoutDirection = value; }
+//        }
+//        [Q_PROPERTY("QSizeF", "size")]
+//        public QSizeF Size {
+//            get { return applet.Size; }
+//            set { applet.Size = value; }
+//        }
+//        [Q_PROPERTY("Qt::FocusPolicy", "focusPolicy")]
+//        public new Qt.FocusPolicy FocusPolicy {
+//            get { return applet.FocusPolicy; }
+//            set { applet.FocusPolicy = value; }
 //        }
-        [Q_PROPERTY("Qt::LayoutDirection", "layoutDirection")]
-        public new Qt.LayoutDirection LayoutDirection {
-            get { return applet.LayoutDirection; }
-            set { applet.LayoutDirection = value; }
-        }
-        [Q_PROPERTY("QSizeF", "size")]
-        public QSizeF Size {
-            get { return applet.Size; }
-            set { applet.Size = value; }
-        }
-        [Q_PROPERTY("Qt::FocusPolicy", "focusPolicy")]
-        public new Qt.FocusPolicy FocusPolicy {
-            get { return applet.FocusPolicy; }
-            set { applet.FocusPolicy = value; }
-        }
         [Q_PROPERTY("bool", "enabled")]
         public bool Enabled {
             get { return applet.Enabled; }
@@ -50,16 +51,16 @@ namespace PlasmaScripting {
             get { return applet.Visible; }
             set { applet.Visible = value; }
         }
-        [Q_PROPERTY("Qt::WindowFlags", "windowFlags")]
-        public uint WindowFlags {
-            get { return applet.WindowFlags; }
-            set { applet.WindowFlags = value; }
-        }
-        [Q_PROPERTY("QString", "windowTitle")]
-        public string WindowTitle {
-            get { return applet.WindowTitle; }
-            set { applet.WindowTitle = value; }
-        }
+//        [Q_PROPERTY("Qt::WindowFlags", "windowFlags")]
+//        public uint WindowFlags {
+//            get { return applet.WindowFlags; }
+//            set { applet.WindowFlags = value; }
+//        }
+//        [Q_PROPERTY("QString", "windowTitle")]
+//        public string WindowTitle {
+//            get { return applet.WindowTitle; }
+//            set { applet.WindowTitle = value; }
+//        }
         public QGraphicsWidget(AppletScript parent) : base(parent) {
             appletScript = parent;
             applet = parent.Applet();
@@ -257,6 +258,12 @@ namespace PlasmaScripting {
         public bool IsWindow() {
             return applet.IsWindow();
         }
+        public QTransform ItemTransform (IQGraphicsItem other) {
+            return applet.ItemTransform(other);
+        }
+        public QTransform ItemTransform (IQGraphicsItem other, ref bool ok) {
+            return applet.ItemTransform(other, ref ok);
+        }
         public QGraphicsItemGroup Group() {
             return applet.Group();
         }
@@ -404,6 +411,9 @@ namespace PlasmaScripting {
         public void MoveBy(double dx, double dy) {
             applet.MoveBy(dx, dy);
         }
+        public double EffectiveOpacity() {
+            return applet.EffectiveOpacity();
+        }
         public void EnsureVisible(QRectF rect, int xmargin, int ymargin) {
             applet.EnsureVisible(rect, xmargin, ymargin);
         }
@@ -485,6 +495,9 @@ namespace PlasmaScripting {
         public QRectF SceneBoundingRect() {
             return applet.SceneBoundingRect();
         }
+        public QPainterPath ClipPath() {
+            return applet.ClipPath();
+        }
         public virtual bool Contains(QPointF point) {
             return applet.Contains(point);
         }
@@ -506,6 +519,9 @@ namespace PlasmaScripting {
         public List<IQGraphicsItem> CollidingItems() {
             return applet.CollidingItems();
         }
+        public bool IsClipped() {
+            return applet.IsClipped();
+        }
         public bool IsObscured() {
             return applet.IsObscured();
         }
@@ -545,6 +561,42 @@ namespace PlasmaScripting {
         public void Scroll(double dx, double dy) {
             applet.Scroll(dx, dy);
         }
+        public QRectF MapRectFromItem (IQGraphicsItem item, double x, double y, double w, double h) {
+            return applet.MapRectFromItem(item, x, y, w, h);
+        }
+        public QRectF MapRectFromItem (IQGraphicsItem item, QRectF rect) {
+            return applet.MapRectFromItem(item, rect);
+        }
+        public QRectF MapRectFromParent (double x, double y, double w, double h) {
+            return applet.MapRectFromParent(x, y, w, h);
+        }
+        public QRectF MapRectFromParent (QRectF rect) {
+            return applet.MapRectFromParent(rect);
+        }
+        public QRectF MapRectFromScene (double x, double y, double w, double h) {
+            return applet.MapRectFromScene(x, y, w, h);
+        }
+        public QRectF MapRectFromScene (QRectF rect) {
+            return applet.MapRectFromScene(rect);
+        }
+        public QRectF MapRectToItem (IQGraphicsItem item, double x, double y, double w, double h) {
+            return applet.MapRectToItem(item, x, y, w, h);
+        }
+        public QRectF MapRectToItem (IQGraphicsItem item, QRectF rect) {
+            return applet.MapRectToItem(item, rect);
+        }
+        public QRectF MapRectToParent (double x, double y, double w, double h) {
+            return applet.MapRectToParent(x, y, w, h);
+        }
+        public QRectF MapRectToParent (QRectF rect) {
+            return applet.MapRectToParent(rect);
+        }
+        public QRectF MapRectToScene (double x, double y, double w, double h) {
+            return applet.MapRectToScene(x, y, w, h);
+        }
+        public QRectF MapRectToScene (QRectF rect) {
+            return applet.MapRectToScene(rect);
+        }
         public QPointF MapToItem(IQGraphicsItem item, QPointF point) {
             return applet.MapToItem(item, point);
         }
@@ -818,6 +870,12 @@ namespace PlasmaScripting {
         public bool IsLayout() {
             return applet.IsLayout();
         }
+        public double Opacity() {
+            return applet.Opacity();
+        }
+        public void SetOpacity (double opacity) {
+            applet.SetOpacity(opacity);
+        }
         protected new IQGraphicsWidgetSignals Emit {
             get { return (IQGraphicsWidgetSignals) Q_EMIT; }
         }
