@@ -1,9 +1,9 @@
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Lesser General Public License as        *
+ *   published by the Free Software Foundation; either version 2 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  ***************************************************************************/
 
@@ -81,6 +81,10 @@
 #include "smokeqyoto.h"
 #include "marshall_macros.h"
 
+#ifdef Q_OS_WIN
+#include <windows.h>
+#endif
+
 extern "C" {
 Q_DECL_EXPORT GetIntPtr ListToPointerList;
 Q_DECL_EXPORT CreateListFn ConstructList;
@@ -105,6 +109,13 @@ Q_DECL_EXPORT AddInt AddIntToListInt;
 Q_DECL_EXPORT AddUInt AddUIntToListUInt;
 Q_DECL_EXPORT AddIntObject AddIntObjectToDictionary;
 
+Q_DECL_EXPORT GetIntPtr QPairGetFirst;
+Q_DECL_EXPORT GetIntPtr QPairGetSecond;
+Q_DECL_EXPORT CreateQPairFn CreateQPair;
+
+Q_DECL_EXPORT SetIntPtr UnboxToStackItem;
+Q_DECL_EXPORT CreateInstanceFn BoxFromStackItem;
+
 Q_DECL_EXPORT GetIntPtr GenericPointerGetIntPtr;
 Q_DECL_EXPORT CreateInstanceFn CreateGenericPointer;
 
@@ -223,6 +234,31 @@ Q_DECL_EXPORT void InstallCreateGenericPointer(CreateInstanceFn callback)
 	CreateGenericPointer = callback;
 }
 
+Q_DECL_EXPORT void InstallQPairGetFirst(GetIntPtr callback)
+{
+	QPairGetFirst = callback;
+}
+
+Q_DECL_EXPORT void InstallQPairGetSecond(GetIntPtr callback)
+{
+	QPairGetSecond = callback;
+}
+
+Q_DECL_EXPORT void InstallCreateQPair(CreateQPairFn callback)
+{
+	CreateQPair = callback;
+}
+
+Q_DECL_EXPORT void InstallUnboxToStackItem(SetIntPtr callback)
+{
+	UnboxToStackItem = callback;
+}
+
+Q_DECL_EXPORT void InstallBoxFromStackItem (CreateInstanceFn callback)
+{
+	BoxFromStackItem = callback;
+}
+
 Q_DECL_EXPORT void* ConstructPointerList()
 {
 	void * list = (void*) new QList<void*>;
@@ -336,7 +372,7 @@ Q_DECL_EXPORT void AddWizardButtonToQList(void* ptr, int i)
 
 }
 
-// extern bool isDerivedFromByName(Smoke *smoke, const char *className, const char *baseClassName);
+// extern bool isDerivedFrom(Smoke *smoke, const char *className, const char *baseClassName);
 extern void mapPointer(void * obj, smokeqyoto_object *o, Smoke::Index classId, void *lastptr);
 
 Q_DECL_EXPORT bool
@@ -350,8 +386,8 @@ IsContainedInstanceQt(smokeqyoto_object *o)
 			|| qstrcmp(className, "QModelIndex") == 0 )
 	{
 		return true;
-	} else if (o->smoke->isDerivedFromByName(className, "QLayoutItem")) {
-		QLayoutItem * item = (QLayoutItem *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QLayoutItem").index);
+	} else if (o->smoke->isDerivedFrom(className, "QLayoutItem")) {
+		QLayoutItem * item = (QLayoutItem *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QLayoutItem", true).index);
 		if (item->layout() != 0 || item->widget() != 0 || item->spacerItem() != 0) {
 			return true;
 		}
@@ -360,35 +396,38 @@ IsContainedInstanceQt(smokeqyoto_object *o)
 		if (item->listWidget() != 0) {
 			return true;
 		}
-	} else if (o->smoke->isDerivedFromByName(className, "QTableWidgetItem")) {
-		QTableWidgetItem * item = (QTableWidgetItem *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QTableWidgetItem").index);
+	} else if (o->smoke->isDerivedFrom(className, "QTableWidgetItem")) {
+		QTableWidgetItem * item = (QTableWidgetItem *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QTableWidgetItem", true).index);
 		if (item->tableWidget() != 0) {
 			return true;
 		}
-	} else if (o->smoke->isDerivedFromByName(className, "QTreeWidgetItem")) {
-		QTreeWidgetItem * item = (QTreeWidgetItem *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QTreeWidgetItem").index);
+	} else if (o->smoke->isDerivedFrom(className, "QTreeWidgetItem")) {
+		QTreeWidgetItem * item = (QTreeWidgetItem *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QTreeWidgetItem", true).index);
 		if (item->treeWidget() != 0) {
 			return true;
 		}
-	} else if (o->smoke->isDerivedFromByName(className, "QWidget")) {
-		QWidget * qwidget = (QWidget *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QWidget").index);
-		if (qwidget->parentWidget() != 0) {
+	} else if (o->smoke->isDerivedFrom(className, "QGraphicsScene")) {
+		QGraphicsScene * scene = (QGraphicsScene *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QGraphicsScene", true).index);
+		if (scene->views().count() > 0 || scene->parent() != 0) {
 			return true;
 		}
-		// Don't garbage collect custom subclasses of QWidget classes for now
-		const QMetaObject * meta = qwidget->metaObject();
-		Smoke::ModuleIndex classId = o->smoke->idClass(meta->className());
-		return (classId.index == 0);
-	} else if (o->smoke->isDerivedFromByName(className, "QObject")) {
-		QObject * qobject = (QObject *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QObject").index);
+	} else if (o->smoke->isDerivedFrom(className, "QWidget")) {
+		// Only garbage collect the widget if it's hidden, doesn't have any parents and if there are no more 
+		// references to it in the code. This should produce a more 'natural' behaviour for top-level widgets.
+		QWidget * qwidget = (QWidget *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QWidget", true).index);
+		if (qwidget->isVisible() || qwidget->parent() != 0) {
+			return true;
+		}
+	} else if (o->smoke->isDerivedFrom(className, "QObject")) {
+		QObject * qobject = (QObject *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QObject", true).index);
 		if (qobject->parent() != 0) {
 			return true;
 		}
-	} else if (o->smoke->isDerivedFromByName(className, "QTextBlockUserData")) {
+	} else if (o->smoke->isDerivedFrom(className, "QTextBlockUserData")) {
 		return true;
-	} else if (o->smoke->isDerivedFromByName(className, "QGraphicsItem")) {
-		QGraphicsItem * item = (QGraphicsItem *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QGraphicsItem").index);
-		if (item->scene() != 0) {
+	} else if (o->smoke->isDerivedFrom(className, "QGraphicsItem")) {
+		QGraphicsItem * item = (QGraphicsItem *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QGraphicsItem", true).index);
+		if (item->scene() != 0 || item->parentItem() != 0) {
 			return true;
 		}
 	}
@@ -403,51 +442,58 @@ IsContainedInstanceQt(smokeqyoto_object *o)
 Q_DECL_EXPORT const char *
 qyoto_resolve_classname_qt(smokeqyoto_object * o)
 {
-	if (o->smoke->isDerivedFromByName(o->smoke->classes[o->classId].className, "QEvent")) {
-		QEvent * qevent = (QEvent *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QEvent").index);
+#define SET_SMOKEQYOTO_OBJECT(className) \
+    { \
+        Smoke::ModuleIndex mi = Smoke::findClass(className); \
+        o->classId = mi.index; \
+        o->smoke = mi.smoke; \
+    }
+
+	if (o->smoke->isDerivedFrom(o->smoke->classes[o->classId].className, "QEvent")) {
+		QEvent * qevent = (QEvent *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QEvent", true).index);
 		switch (qevent->type()) {
 		case QEvent::Timer:
-			o->classId = o->smoke->idClass("QTimerEvent").index;
+   			SET_SMOKEQYOTO_OBJECT("QTimerEvent")
 			break;
 		case QEvent::MouseButtonPress:
 		case QEvent::MouseButtonRelease:
 		case QEvent::MouseButtonDblClick:
 		case QEvent::MouseMove:
-			o->classId = o->smoke->idClass("QMouseEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QMouseEvent")
 			break;
 		case QEvent::KeyPress:
 		case QEvent::KeyRelease:
 		case QEvent::ShortcutOverride:
-			o->classId = o->smoke->idClass("QKeyEvent").index;
+   			SET_SMOKEQYOTO_OBJECT("QKeyEvent")
 			break;
 		case QEvent::FocusIn:
 		case QEvent::FocusOut:
-			o->classId = o->smoke->idClass("QFocusEvent").index;
+   			SET_SMOKEQYOTO_OBJECT("QFocusEvent")
 			break;
 		case QEvent::Enter:
 		case QEvent::Leave:
-			o->classId = o->smoke->idClass("QEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QEvent")
 			break;
 		case QEvent::Paint:
-			o->classId = o->smoke->idClass("QPaintEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QPaintEvent")
 			break;
 		case QEvent::Move:
-			o->classId = o->smoke->idClass("QMoveEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QMoveEvent")
 			break;
 		case QEvent::Resize:
-			o->classId = o->smoke->idClass("QResizeEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QResizeEvent")
 			break;
 		case QEvent::Create:
 		case QEvent::Destroy:
-			o->classId = o->smoke->idClass("QEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QEvent")
 			break;
 		case QEvent::Show:
-			o->classId = o->smoke->idClass("QShowEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QShowEvent")
 			break;
 		case QEvent::Hide:
-			o->classId = o->smoke->idClass("QHideEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QHideEvent")
 		case QEvent::Close:
-			o->classId = o->smoke->idClass("QCloseEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QCloseEvent")
 			break;
 		case QEvent::Quit:
 		case QEvent::ParentChange:
@@ -457,10 +503,10 @@ qyoto_resolve_classname_qt(smokeqyoto_object * o)
 		case QEvent::WindowDeactivate:
 		case QEvent::ShowToParent:
 		case QEvent::HideToParent:
-			o->classId = o->smoke->idClass("QEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QEvent")
 			break;
 		case QEvent::Wheel:
-			o->classId = o->smoke->idClass("QWheelEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QWheelEvent")
 			break;
 		case QEvent::WindowTitleChange:
 		case QEvent::WindowIconChange:
@@ -469,36 +515,36 @@ qyoto_resolve_classname_qt(smokeqyoto_object * o)
 		case QEvent::ApplicationLayoutDirectionChange:
 		case QEvent::ApplicationPaletteChange:
 		case QEvent::PaletteChange:
-			o->classId = o->smoke->idClass("QEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QEvent")
 			break;
 		case QEvent::Clipboard:
-			o->classId = o->smoke->idClass("QClipboardEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QClipboardEvent")
 			break;
 		case QEvent::Speech:
 		case QEvent::MetaCall:
 		case QEvent::SockAct:
 		case QEvent::WinEventAct:
 		case QEvent::DeferredDelete:
-			o->classId = o->smoke->idClass("QEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QEvent")
 			break;
 		case QEvent::DragEnter:
-			o->classId = o->smoke->idClass("QDragEnterEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QDragEnterEvent")
 			break;
 		case QEvent::DragLeave:
-			o->classId = o->smoke->idClass("QDragLeaveEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QDragLeaveEvent")
 			break;
 		case QEvent::DragMove:
-			o->classId = o->smoke->idClass("QDragMoveEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QDragMoveEvent")
 		case QEvent::Drop:
-			o->classId = o->smoke->idClass("QDropEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QDropEvent")
 			break;
 		case QEvent::DragResponse:
-			o->classId = o->smoke->idClass("QDragResponseEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QDragResponseEvent")
 			break;
 		case QEvent::ChildAdded:
 		case QEvent::ChildRemoved:
 		case QEvent::ChildPolished:
-			o->classId = o->smoke->idClass("QChildEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QChildEvent")
 			break;
 		case QEvent::ShowWindowRequest:
 		case QEvent::PolishRequest:
@@ -508,21 +554,24 @@ qyoto_resolve_classname_qt(smokeqyoto_object * o)
 		case QEvent::EmbeddingControl:
 		case QEvent::ActivateControl:
 		case QEvent::DeactivateControl:
-			o->classId = o->smoke->idClass("QEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QEvent")
 			break;
-		case QEvent::ContextMenu:
-			o->classId = o->smoke->idClass("QContextMenuEvent").index;
+        case QEvent::ContextMenu:
+			SET_SMOKEQYOTO_OBJECT("QContextMenuEvent")
+			break;
+  case QEvent::DynamicPropertyChange:
+			SET_SMOKEQYOTO_OBJECT("QDynamicPropertyChangeEvent")
 			break;
 		case QEvent::InputMethod:
-			o->classId = o->smoke->idClass("QInputMethodEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QInputMethodEvent")
 			break;
 		case QEvent::AccessibilityPrepare:
-			o->classId = o->smoke->idClass("QEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QEvent")
 			break;
 		case QEvent::TabletMove:
 		case QEvent::TabletPress:
 		case QEvent::TabletRelease:
-			o->classId = o->smoke->idClass("QTabletEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QTabletEvent")
 			break;
 		case QEvent::LocaleChange:
 		case QEvent::LanguageChange:
@@ -530,10 +579,10 @@ qyoto_resolve_classname_qt(smokeqyoto_object * o)
 		case QEvent::Style:
 		case QEvent::OkRequest:
 		case QEvent::HelpRequest:
-			o->classId = o->smoke->idClass("QEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QEvent")
 			break;
 		case QEvent::IconDrag:
-			o->classId = o->smoke->idClass("QIconDragEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QIconDragEvent")
 			break;
 		case QEvent::FontChange:
 		case QEvent::EnabledChange:
@@ -542,36 +591,36 @@ qyoto_resolve_classname_qt(smokeqyoto_object * o)
 		case QEvent::IconTextChange:
 		case QEvent::ModifiedChange:
 		case QEvent::MouseTrackingChange:
-			o->classId = o->smoke->idClass("QEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QEvent")
 			break;
 		case QEvent::WindowBlocked:
 		case QEvent::WindowUnblocked:
 		case QEvent::WindowStateChange:
-			o->classId = o->smoke->idClass("QWindowStateChangeEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QWindowStateChangeEvent")
 			break;
 		case QEvent::ToolTip:
 		case QEvent::WhatsThis:
-			o->classId = o->smoke->idClass("QHelpEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QHelpEvent")
 			break;
 		case QEvent::StatusTip:
-			o->classId = o->smoke->idClass("QEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QEvent")
 			break;
 		case QEvent::ActionChanged:
 		case QEvent::ActionAdded:
 		case QEvent::ActionRemoved:
-			o->classId = o->smoke->idClass("QActionEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QActionEvent")
 			break;
 		case QEvent::FileOpen:
-			o->classId = o->smoke->idClass("QFileOpenEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QFileOpenEvent")
 			break;
 		case QEvent::Shortcut:
-			o->classId = o->smoke->idClass("QShortcutEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QShortcutEvent")
 			break;
 		case QEvent::WhatsThisClicked:
-			o->classId = o->smoke->idClass("QWhatsThisClickedEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QWhatsThisClickedEvent")
 			break;
 		case QEvent::ToolBarChange:
-			o->classId = o->smoke->idClass("QToolBarChangeEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QToolBarChangeEvent")
 			break;
 		case QEvent::ApplicationActivated:
 		case QEvent::ApplicationDeactivated:
@@ -579,129 +628,95 @@ qyoto_resolve_classname_qt(smokeqyoto_object * o)
 		case QEvent::EnterWhatsThisMode:
 		case QEvent::LeaveWhatsThisMode:
 		case QEvent::ZOrderChange:
-			o->classId = o->smoke->idClass("QEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QEvent")
 			break;
 		case QEvent::HoverEnter:
 		case QEvent::HoverLeave:
 		case QEvent::HoverMove:
-			o->classId = o->smoke->idClass("QHoverEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QHoverEvent")
 			break;
 		case QEvent::AccessibilityHelp:
 		case QEvent::AccessibilityDescription:
-			o->classId = o->smoke->idClass("QEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QEvent")
 #if QT_VERSION >= 0x40200
 		case QEvent::GraphicsSceneMouseMove:
 		case QEvent::GraphicsSceneMousePress:
 		case QEvent::GraphicsSceneMouseRelease:
 		case QEvent::GraphicsSceneMouseDoubleClick:
-			o->classId = o->smoke->idClass("QGraphicsSceneMouseEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsSceneMouseEvent")
 			break;
 		case QEvent::GraphicsSceneContextMenu:
-			o->classId = o->smoke->idClass("QGraphicsSceneContextMenuEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsSceneContextMenuEvent")
 			break;
 		case QEvent::GraphicsSceneHoverEnter:
 		case QEvent::GraphicsSceneHoverMove:
 		case QEvent::GraphicsSceneHoverLeave:
-			o->classId = o->smoke->idClass("QGraphicsSceneHoverEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsSceneHoverEvent")
 			break;
 		case QEvent::GraphicsSceneHelp:
-			o->classId = o->smoke->idClass("QGraphicsSceneHelpEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsSceneHelpEvent")
 			break;
 		case QEvent::GraphicsSceneDragEnter:
 		case QEvent::GraphicsSceneDragMove:
 		case QEvent::GraphicsSceneDragLeave:
 		case QEvent::GraphicsSceneDrop:
-			o->classId = o->smoke->idClass("QGraphicsSceneDragDropEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsSceneDragDropEvent")
 			break;
 		case QEvent::GraphicsSceneWheel:
-			o->classId = o->smoke->idClass("QGraphicsSceneWheelEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsSceneWheelEvent")
 			break;
 		case QEvent::KeyboardLayoutChange:
-			o->classId = o->smoke->idClass("QEvent").index;
+			SET_SMOKEQYOTO_OBJECT("QEvent")
 			break;
 #endif
 		default:
 			break;
 		}
-	} else if (o->smoke->isDerivedFromByName(o->smoke->classes[o->classId].className, "QObject")) {
-		QObject * qobject = (QObject *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QObject").index);
-		const QMetaObject * meta = qobject->metaObject();
-		const char * oldClassName = o->smoke->classes[o->classId].className;
-		while (meta != 0) {
-			o->smoke = Smoke::classMap[meta->className()];
-			if (o->smoke != 0) {
-				o->classId = o->smoke->idClass(meta->className()).index;
-				if (o->classId != 0) {
-					// if the classname is different, resolve the new name again - otherwise just 'C#-ify' it.
-					if (strcmp(oldClassName, o->smoke->className(o->classId)) == 0) {
-						if (strcmp(o->smoke->classes[o->classId].className, "QAbstractItemModel") == 0)
-							return "Qyoto.QItemModel";
-						if (strcmp(o->smoke->classes[o->classId].className, "QAbstractButton") == 0)
-							return "Qyoto.QAbstractButtonInternal";
-						if (strcmp(o->smoke->classes[o->classId].className, "QAbstractProxyModel") == 0)
-							return "Qyoto.QAbstractProxyModelInternal";
-						if (strcmp(o->smoke->classes[o->classId].className, "QAbstractItemDelegate") == 0)
-							return "Qyoto.QAbstractItemDelegateInternal";
-						if (strcmp(o->smoke->classes[o->classId].className, "QAbstractItemView") == 0)
-							return "Qyoto.QAbstractItemViewInternal";
-						if (strcmp(o->smoke->classes[o->classId].className, "QAbstractListModel") == 0)
-							return "Qyoto.QAbstractListModelInternal";
-						if (strcmp(o->smoke->classes[o->classId].className, "QAbstractTextDocumentLayout") == 0)
-							return "Qyoto.QAbstractTextDocumentLayoutInternal";
-						if (strcmp(o->smoke->classes[o->classId].className, "QLayout") == 0)
-							return "Qyoto.QLayoutInternal";
-						return qyoto_modules[o->smoke].binding->className(o->classId);
-					} else {
-						return qyoto_resolve_classname(o);
-					}
-				}
-			}
-
-			meta = meta->superClass();
-		}
-	} else if (o->smoke->isDerivedFromByName(o->smoke->classes[o->classId].className, "QGraphicsItem")) {
-		QGraphicsItem * item = (QGraphicsItem *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QGraphicsItem").index);
+	} else if (o->smoke->isDerivedFrom(o->smoke->classes[o->classId].className, "QGraphicsItem")) {
+		QGraphicsItem * item = (QGraphicsItem *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QGraphicsItem", true).index);
 		switch (item->type()) {
 		case 1:
-			o->classId = o->smoke->idClass("QGraphicsItem").index;
-			return "Qyoto.QGraphicsItemInternal";
+			SET_SMOKEQYOTO_OBJECT("QGraphicsItem")
+			break;
 		case 2:
-			o->classId = o->smoke->idClass("QGraphicsPathItem").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsPathItem")
 			break;
 		case 3:
-			o->classId = o->smoke->idClass("QGraphicsRectItem").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsRectItem")
 		case 4:
-			o->classId = o->smoke->idClass("QGraphicsEllipseItem").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsEllipseItem")
 			break;
 		case 5:
-			o->classId = o->smoke->idClass("QGraphicsPolygonItem").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsPolygonItem")
 			break;
 		case 6:
-			o->classId = o->smoke->idClass("QGraphicsLineItem").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsLineItem")
 			break;
 		case 7:
-			o->classId = o->smoke->idClass("QGraphicsItem").index;
-			return "Qyoto.QGraphicsItemInternal";
+			SET_SMOKEQYOTO_OBJECT("QGraphicsItem")
+			break;
 		case 8:
-			o->classId = o->smoke->idClass("QGraphicsTextItem").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsTextItem")
 			break;
 		case 9:
-			o->classId = o->smoke->idClass("QGraphicsSimpleTextItem").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsSimpleTextItem")
 			break;
 		case 10:
-			o->classId = o->smoke->idClass("QGraphicsItemGroup").index;
+			SET_SMOKEQYOTO_OBJECT("QGraphicsItemGroup")
 			break;
 		}
-	} else if (o->smoke->isDerivedFromByName(o->smoke->classes[o->classId].className, "QLayoutItem")) {
-		QLayoutItem * item = (QLayoutItem *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QLayoutItem").index);
+	} else if (o->smoke->isDerivedFrom(o->smoke->classes[o->classId].className, "QLayoutItem")) {
+		QLayoutItem * item = (QLayoutItem *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QLayoutItem", true).index);
 		if (item->widget() != 0) {
-			o->classId = o->smoke->idClass("QWidgetItem").index;
+			SET_SMOKEQYOTO_OBJECT("QWidgetItem")
 		} else if (item->spacerItem() != 0) {
-			o->classId = o->smoke->idClass("QSpacerItem").index;
+			SET_SMOKEQYOTO_OBJECT("QSpacerItem")
 		}
 	}
 	
 	return qyoto_modules[o->smoke].binding->className(o->classId);
+
+#undef SET_SMOKEQYOTO_OBJECT
 }
 
 bool
@@ -717,16 +732,21 @@ construct_copy(smokeqyoto_object *o)
 {
     const char *className = o->smoke->className(o->classId);
     int classNameLen = strlen(className);
-    char *ccSig = new char[classNameLen + 2];       // copy constructor signature
-    strcpy(ccSig, className);
-    strcat(ccSig, "#");
+
+    // copy constructor signature
+    QByteArray ccSig(className);
+    int pos = ccSig.lastIndexOf("::");
+    if (pos != -1) {
+        ccSig = ccSig.mid(pos + strlen("::"));
+    }
+    ccSig.append("#");
+
     Smoke::ModuleIndex ccId = o->smoke->findMethodName(className, ccSig);
-    delete[] ccSig;
 
     char *ccArg = new char[classNameLen + 8];
     sprintf(ccArg, "const %s&", className);
 
-    Smoke::ModuleIndex classId = { o->smoke, o->classId };
+    Smoke::ModuleIndex classId(o->smoke, o->classId);
     Smoke::ModuleIndex ccMeth = o->smoke->findMethod(classId, ccId);
 
     if(!ccMeth.index) {
@@ -784,27 +804,34 @@ StringArrayToCharStarStar(int length, char ** strArray)
 }
 
 Q_DECL_EXPORT void *
-StringToQString(char *str)
+StringToQString(const ushort *str)
 {
-	QString * result = new QString(QString::fromUtf8(str));
+	QString * result = new QString(QString::fromUtf16(str));
 	return (void *) result;
 }
 
-Q_DECL_EXPORT char *
+Q_DECL_EXPORT const ushort *
 StringFromQString(void *ptr)
 {
-    QByteArray ba = ((QString *) ptr)->toUtf8();
-    return strdup(ba.constData());
+	QString* str = (QString*) ptr;
+	int len = str->length() + 1; // include the terminating \0
+#ifdef Q_OS_WIN
+	ushort *copy = (ushort*) GlobalAlloc(GMEM_FIXED, sizeof(ushort) * len);
+#else
+	ushort *copy = (ushort*) malloc(sizeof(ushort) * len);
+#endif
+	memcpy(copy, str->utf16(), sizeof(ushort) * len);
+	// return a copy of the string - the runtime will take ownership of it and care about deletion
+	return copy;
 }
 
 Q_DECL_EXPORT void *
 StringArrayToQStringList(int length, char ** strArray)
 {
 	QStringList * result = new QStringList();
-	char ** ca = (char**) StringArrayToCharStarStar(length, strArray);
 	
 	for (int i = 0; i < length; i++) {
-		(*result) << QString::fromUtf8(ca[i]);
+		(*result) << QString::fromUtf8(strArray[i]);
 	}
 	return (void*) result;
 }
@@ -1038,10 +1065,10 @@ marshall_basetype(Marshall *m)
 		}
 
 		if(m->type().isStack()) {
-// 		    o->allocated = true;
-			// Keep a mapping of the pointer so that it is only wrapped once
-		    mapPointer(obj, o, o->classId, 0);
+		    o->allocated = true;
 		}
+		// Keep a mapping of the pointer so that it is only wrapped once
+		mapPointer(obj, o, o->classId, 0);
 		
 		m->var().s_class = obj;
 	}
@@ -1089,7 +1116,7 @@ static void marshall_charP(Marshall *m) {
 			return;
 		}
 	    if (p != 0) {
-			m->var().s_class = (*IntPtrFromCharStar)(strdup(p));
+			m->var().s_class = (*IntPtrFromCharStar)(p);
 	    } else {
 			m->var().s_class = 0;
 		}
@@ -1170,7 +1197,7 @@ static void marshall_QString(Marshall *m) {
 				m->var().s_class = (*IntPtrFromQString)(s);
 			}
 
-			if (m->cleanup())
+			if (m->cleanup() || m->type().isStack())
 				delete s;
 			} else {
 				m->var().s_voidp = 0;
@@ -1450,9 +1477,9 @@ void marshall_QDBusVariant(Marshall *m) {
 
 		if (m->type().isStack()) {
 		    o->allocated = true;
-			// Keep a mapping of the pointer so that it is only wrapped once
-		    mapPointer(obj, o, o->classId, 0);
 		}
+		// Keep a mapping of the pointer so that it is only wrapped once
+		mapPointer(obj, o, o->classId, 0);
 		
 		m->var().s_class = obj;
 	}
@@ -1878,6 +1905,8 @@ DEF_VALUELIST_MARSHALLER( QNetworkCookieList, QList<QNetworkCookie>, QNetworkCoo
 DEF_VALUELIST_MARSHALLER( QPrinterInfoList, QList<QPrinterInfo>, QPrinterInfo )
 #endif
 
+DEF_QPAIR_MARSHALLER( QPair_QHostAddress_int, QHostAddress, int, "Qyoto.QHostAddress", "System.Int32" )
+
 Q_DECL_EXPORT TypeHandler Qyoto_handlers[] = {
     { "bool*", marshall_boolR },
     { "bool&", marshall_boolR },
@@ -1953,6 +1982,8 @@ Q_DECL_EXPORT TypeHandler Qyoto_handlers[] = {
     { "QModelIndexList&", marshall_QModelIndexList },
     { "QObjectList", marshall_QObjectList },
     { "QObjectList&", marshall_QObjectList },
+    { "QPair<QHostAddress,int>", marshall_QPair_QHostAddress_int },
+    { "QPair<QHostAddress,int>&", marshall_QPair_QHostAddress_int },
     { "qreal*", marshall_doubleR },
     { "qreal&", marshall_doubleR },
     { "QStringList", marshall_QStringList },
@@ -2058,3 +2089,5 @@ Marshall::HandlerFn getMarshallFn(const SmokeType &type) {
 
 	return marshall_unknown;
 }
+
+// kate: space-indent off; mixed-indent off;
